/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#ifndef ANNOTATION_HPP
#define ANNOTATION_HPP

#include <QString>
#include <DoQuBook/DoQuBook_global.hpp>
#include <DoQuBook/AnnotationVisitor.hpp>
#include <QSharedPointer>

#include <QVariant>
#include <DoQuBook/SimpleMarshallable.hpp>
#include <iostream>
#include <DoQuBook/Singleton.hpp>
#include <DoQuBook/Defect.hpp>

namespace DoQuBook
{

    /**
      * Classe bouchon en attendant un service web sur DoQuBookWeb.
      */
    class DefectsServices : public Singleton<DefectsServices>
    {
            friend class Singleton<DefectsServices>;
            friend class Defect;
        public :

            DefectsServices() {
                /* Global Boolean Values */
                _allDefects.append (Defects::Global_IsWhitePage);
                /* Global Int Values */
                _allDefects.append (Defects::Global_OverAllQuality_Perception);
                _allDefects.append (Defects::Global_TextSizeVariation_Perception);
                _allDefects.append (Defects::Global_QuantityText_Perception);
                _allDefects.append (Defects::Global_QuantityImages_Perception);
                _allDefects.append (Defects::Global_QuantityBleedThrough_Perception);
                _allDefects.append (Defects::Global_QuantitySpots_Perception);
                _allDefects.append (Defects::Global_QualityText_Perception);
                /* Local Int Values Annotations */
                _allDefects.append (Defects::Local_BleedThrough_Perception);
                _allDefects.append (Defects::Local_Spot_Perception);
                /* Global Registration parameters */
                _allDefects.append (Defects::Global_Registration_X);
                _allDefects.append (Defects::Global_Registration_Y);
                _allDefects.append (Defects::Global_Registration_Teta);
                /* Recto - Verso */
                _allDefects.append (Defects::Verso);
                /* Global Trinarization */
                _allDefects.append (Defects::Global_BleedThroughGrayScale_Ink);
                _allDefects.append (Defects::Global_BleedThroughGrayScale_Background);
                _allDefects.append (Defects::Local_BleedThroughGrayScale_Ink);
                _allDefects.append (Defects::Local_BleedThroughGrayScale_Background);
                /* Local Trinarization */
                _perceptionLocalDefects .append (Defects::Local_Spot_Perception);
                _perceptionLocalDefects .append (Defects::Local_BleedThrough_Perception);

                _perceptionSortedNames.append ("None");
                _perceptionSortedNames.append ("Low");
                _perceptionSortedNames.append ("Medium");
                _perceptionSortedNames.append ("High");
                _perceptionSortedNames.append ("Super High");

                _perceptionValues["None"] = 0;
                _perceptionValues["Low"] = 1;
                _perceptionValues["Medium"] = 2;
                _perceptionValues["High"] = 3;
                _perceptionValues["Super High"] = 4;
            }
            const QString defectName (Defect d) {
                return d.getName();
            }
            QVariant::Type defectType (Defect d) {
                return d.getType();
            }

            const QStringList defects() {
                QStringList l;
                foreach (Defect  d , _allDefects) {
                    l << d.getName();
                }
                return l;
            }

            const Defect defect (QString d) {
                bool found = false;
                Defect defect = _allDefects.first();
                QListIterator<Defect> it (_allDefects);
                while (!found && it.hasNext() ) {
                    defect = it.next();
                    if (defect.getName() == d) {
                        found = true;
                    }
                }
                if (found) {
                    return defect;
                }
                else {
                    throw QString ("Defect of name " + d + "does not exists");
                }
            }

            const QList<Defect> perceptionLocalDefects() {
                return _perceptionLocalDefects;
            }

            const QStringList perceptionLocalDefectsNames() {
                QStringList l;
                foreach (Defect d, _perceptionLocalDefects) {
                    l << defectName (d);
                }
                return l;
            }

            const QStringList perceptionSortedClassNames() {
                return _perceptionSortedNames;
            }
            int valueOfClass (QString s) {
                return _perceptionValues[s];
            }

        private :
            void addDefect (Defect d) {
                _allDefects.append (d);
            }

        private :
            QList<Defect> _allDefects;
            QList<Defect> _perceptionLocalDefects;
            QStringList _perceptionSortedNames;
            QMap<QString, int> _perceptionValues;
    };

    class Annotation;
    typedef QSharedPointer<Annotation> AnnotationPtr;

    /*!
      *
      * \class Annotation
      * \brief An abstract class with a name attribut for annotations
      *
      * The class provides access to the name of an annotation
      *  to provide a new type of annotations, it's a good start to inherit from Annotation.
      */
    class DoQuBookSHARED_EXPORT Annotation  : public SimpleMarshallable
    {
            Q_OBJECT

            Q_PROPERTY (int id READ getId WRITE setId)
            Q_PROPERTY (QString name READ getName WRITE setName)
            Q_PROPERTY (QString username READ getUserName WRITE setUserName)
            Q_PROPERTY (QString description READ getDescription WRITE setDescription)
            Q_PROPERTY (int version READ getVersion WRITE setVersion)
            Q_PROPERTY (bool groundTruth READ isGroundTruth WRITE setIsGroundTruth)
        public:

            MARSHALLABLE_DECLARE_SEE_ALSO()

            MARSHALLABLE_CREATE_FONCTIONS (Annotation)

            QString getName() const {
                return _name;
            }
            void setName (const QString & name) {
                _name = name;
                emitModified();
            }

            virtual void accept (AnnotationVisitor & /*visitor*/) const {}

            virtual bool equals (const Annotation & autre) {
                return (_name.toStdString() == autre._name.toStdString() );
            }

            virtual void merge (MarshallablePtr ann);


            QString getUserName() const;

        public slots:

            void setUserName (QString arg);

            void setDescription (const QString & arg) {
                m_description = arg;
            }

            void setId (int arg) {
                m_id = arg;
            }

            void setIsGroundTruth (bool arg) {
                m_groundTruth = arg;
            }

            void setVersion (int arg) {
                m_version = arg;
            }

        signals:
            void modified();

        public :
            Annotation (const QString & name);
            Annotation ();

            QString getDescription() const {
                return m_description;
            }

            int getId() const {
                return m_id;
            }

            bool isGroundTruth() const {
                return m_groundTruth;
            }

            long getVersion() const {
                return m_version;
            }

        protected:
            int m_id;
            QString _name;
            QString _username;
            QString m_description;
            bool m_groundTruth;
            int m_version;
    };

} // namespace

Q_DECLARE_METATYPE ( DoQuBook::AnnotationPtr )

#endif // ANNOTATION_HPP
