/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#ifndef ANNOTATIONLOCAL_HPP
#define ANNOTATIONLOCAL_HPP

#include <DoQuBook/Annotation.hpp>
#include <QList>
#include <DoQuBook/AnnotatedPolygon.hpp>
#include <QVariant>


namespace DoQuBook
{

    class AnnotationLocal;
    typedef QSharedPointer<AnnotationLocal> AnnotationLocalPtr;

    /*! \class AnnotationLocal
     * \brief A local annotation with annotated polygons
     *
     * The class provides access to the annotated polygons of
     * a local annotation
     */
    class DoQuBookSHARED_EXPORT AnnotationLocal : public Annotation
    {
            Q_OBJECT

            Q_PROPERTY (QList<AnnotatedPolygonPtr> annotatedPolygons READ getAnnotatedPolygons)
            /// This property is duplicated in all Annotation SubClass (Introspection issues)
            /// if property type is in Annotation : type == Annotation ... and we want the real type
            Q_PROPERTY (QString type READ getType WRITE setType)

        public:

            MARSHALLABLE_EXTENDS (Annotation)

            MARSHALLABLE_CREATE_FONCTIONS (AnnotationLocal)

            static AnnotationLocalPtr create (const QString & name);

            QList<AnnotatedPolygonPtr> getAnnotatedPolygons() const {
                return _annotatedPolygons;
            }
            void setAnnotatedPolygons (QList<AnnotatedPolygonPtr> ann);
            AnnotatedPolygonPtr getAnnotatedPolygon (int i) const {
                return _annotatedPolygons[i];
            }

            /**
              * Adds the polygon IF :
              *    - id > -1,
              *          no existing polygon of same id
              *             if no existing polygon of same location
              *                  ADD
              *              else replace existing id, and value
              *           else  replace existing value and points
              *
              *    - id == -1, no existing polygon with same location
              */
            void addAnnotatedPolygon (AnnotatedPolygonPtr annotatedPolygon);
            void removeAnnotatedPolygon (AnnotatedPolygonPtr annotatedPolygon);

            virtual void accept (AnnotationVisitor & visitor) const;

            virtual QStringList customProperties() {
                QStringList l;
                l << "annotatedPolygons";
                return l;
            }

            virtual bool equals (const Annotation & autre);

            /**
              * Merges two LocalAnnotations.
              * - Merge with incomming annotation (existing polygons) :
              *   For each polygons, checks in the given annotation if there is a similar polygon (points and value matching).
              *   If it is the case, the polygon is merged with the one found in the other annotation.
              *
              * - Adds the other polygons in the given annotation to this annotation if their are not similar to any existing ones.
              */
            virtual void merge (MarshallablePtr ann);

            AnnotatedPolygonPtr findAnnotatedPolygon (AnnotatedPolygonPtr poly);
            AnnotatedPolygonPtr findAnnotatedByPointsMatching (AnnotatedPolygonPtr poly);
            AnnotatedPolygonPtr findAnnotatedPolygonById (int id);

        public slots :

            void convert_annotatedPolygons (QVariantList * list) ;

            void from_annotatedPolygons (QVariant v);

            void setType (QString arg) {
                m_type = arg;
            }

        public:
            friend class Page;
            AnnotationLocal (const QString & name = "");

            QString getType() const {
                return m_type;
            }

        private:
            QList<AnnotatedPolygonPtr> _annotatedPolygons;
            QString m_type;
    };

} // namespace

#endif // ANNOTATIONLOCAL_HPP
