/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#ifndef APPLICATIONCONTEXT_HPP
#define APPLICATIONCONTEXT_HPP


#if QT_VERSION >= 0x050000
  #include <QStandardPaths>
#else
  #include <QDesktopServices>
#endif

#include <DoQuBook/Singleton.hpp>
#include <DoQuBook/Repository.hpp>
#include <DoQuBook/DoQuBook_global.hpp>

namespace DoQuBook
{

    /**
      * The aim of this class is to save and read the
      *  user informations such as its created repositories etc...
      *
      * For now, only repositories are saved and read.
      *
      * This class uses QSettings to save user informations as User System Preferences.
      * ise its creates and read a file  stored in :
      * LINUX : $HOME/.config/labri/DoQuBook.conf
      * MAC : $HOME/Library/Preferences/com.labri.DoQuBook.plist
      * Windows : HKEY_CURRENT_USER\Software\labri\DoQuBook
      *
      * \attention This class does not use a cache meaning that
      *      any call to a method will access the disk.
      */
    class DoQuBookSHARED_EXPORT ApplicationContext : public Singleton<ApplicationContext>
    {
            friend class Singleton<ApplicationContext>;

        public :
            /**
              * Gets the repositories given in parameters to the user preferences
              */
            QList<RepositoryPtr> getRepositories();
            RepositoryPtr getDefaultRepository();
            RepositoryPtr getRepository (const QString & name);
            QStringList getRepositoriesNameList();
            void addRepository (RepositoryPtr repo);

            QString getUserName();
            void setUserName (const QString & username);


            QString getDoQuBookDataPath();
            QString getAppDataPath();
            QString getTempRepoPath();

            qint64 getCacheSize() {
                return 10000000000;
            }
            QNetworkDiskCache * getCache();
            QString pathToCacheDirectory() {
#if QT_VERSION >= 0x050000
                return QStandardPaths::writableLocation (QStandardPaths::CacheLocation) + "/doqubookcpp";
#else
                return QDesktopServices::storageLocation (QDesktopServices::CacheLocation) + "/doqubookcpp";
#endif
            }
            void clearCache();

            bool cacheActivated();
            void activateCache (bool a = true);

            void deactivateInternetPlugins();
            void activateInternetPlugins();
            bool isInternetPluginsActivated();

        private :
            ApplicationContext();
            /*!
              * Saves the repositories given in parameters to the user preferences
              */
            void saveRepos (const QList<RepositoryPtr> &repos);
        private:

            QString organisationName;
            QString appName;
            const static QString DEFAULT_REPOSITORY_NAME;
            bool _cacheActivated;
            bool _internetPluginsActivated;

    };
}

#endif // APPLICATIONCONTEXT_HPP
