/*!
 * \file Book.hpp
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#ifndef BOOK_HPP
#define BOOK_HPP

#include <QObject>
#include <QList>
#include <QString>
#include <DoQuBook/Page.hpp>
#include <DoQuBook/DoQuBook_global.hpp>
#include <DoQuBook/Marshallable.hpp>


/*!
  * \page DoQuBook Introduction to DoQuBook.
  *
  * \section doqubookApp  Creating a DoQuBookApp
  *
  * Directory structure :
  * \code
   mkdir -p ExampleTest/src
   touch ExampleTest/CMakeList.txt
   touch ExampleTest/src/main.cpp
   cp -r $PATH_TO_DOQUBOOK/cmake ExampleTest/
  * \endcode
  * Copy past the folowing in CMakeLists.txt
  * \code
  *
cmake_minimum_required(VERSION 2.8)

SET(CMAKE_MODULE_PATH  "${CMAKE_SOURCE_DIR}/../cmake/Modules;${CMAKE_MODULE_PATH}" ) ## Ok

include(doqubook_functions)

SET(TARGET ExampleTest)

if(APPLE)
    SET(MY_OWN_INSTALL_PREFIX "/Applications" CACHE PATH "Prefix prepended to install directories")
    SET(CMAKE_INSTALL_PREFIX "${MY_OWN_INSTALL_PREFIX}" CACHE INTERNAL "Prefix prepended to install directories" FORCE)
endif(APPLE)

SET (SOURCES
    ./src/main.cpp
)

doqubook_app(ExampleTest
    ${SOURCES}
    DoQuBook_Icon.icns
)
  * \endcode
  * In main.cpp :
  * \code
#include <DoQuBook/DoQuBookApp.hpp>

QString orgName ("labri");

/// Must be the name of the target in CMake
QString appName ("ExampleTest");


class ExampleTest : public DoQuBook::DoQuBookApp
{
public :
    ExampleTest(QCoreApplication &app)
        : DoQuBook::DoQuBookApp(app, orgName, appName)
    {}
protected :
    virtual void run()
    {
       // Your own code goes here
    }
};

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);

    ExampleTest test (app);

    return test.exec();
}

  * \endcode
  * Compilation :
  * \code
  * cd ExampleTest/build
  * cmake ..
  * make -j 5
  * \endcode
  * \section retBook Retreiving a Book.
  *
  *  If the user has used this machine to create some book repositories,
  *  we need to get the wanted repository using the class DoQuBook::ApplicationContext.
  *  Once the repository is created we retreive the associated book provider :
  *  \code
/// The name of the repo provided in DoQuAnnotator
QString repoName = "MyRepo";
/// Get the repository
DoQuBook::RepositoryPtr repo = DoQuBook::ApplicationContext::instance()->getRepository(repoName);
/// And the associated book provider.
DoQuBook::BookProviderPtr provider = repo->getProvider();
     \endcode
     Once the provider is created :
     \code
 /// Using the provider we retreive the books ids :
 QList<int> bookIds = provider->getAllBooks ();
 foreach(int id, bookIds)
 {
    BookPtr b = provider->get(id);
    qDebug() << b->getName();
 }
     \endcode
     As you can see, we do not need to care about the type of repository (local or server or other).

     \section accesPage Access to book pages
     Once a book is retrieved, we can apply an algorithm to all book's pages :
     \code
 foreach(PagePtr page, b->getPages())
 {
    QImage image = page->getImage();
    // apply your algorithm.
 }
     \endcode
     Here also, we do not need to know whether the image is on a server or on the local filesystem.

     \section createProvider Adding a new repository In C++
  *
  *  Repository can be created with the folowing code :
     \code
     // The Repository
        RepositoryPtr localDev = Repository::create();
        localDev->setName("LocalHostDev");

      // The book provider class name :
        QString providerName = "DoQuBook::BookProviderQHttp";

      // The path to a directory containing the books;
        QVariant url = "http://localhost::8080";

      // Create the provider :
        DoQuBook::BookProviderPtr provider = DoQuBook::BookProviderRegistry::instance()->create(providerName, urm);

      // Add it to the repository
        localDev->setProvider(provider);

      // add the new repository to the application preference file :
        ApplicationContext::instance()->addRepository(localDev);

     \endcode
     Note that once the repository is added to the application context, we can retreive its
        parametters just by giving its name to the function
    \code
        DoQuBook::RepositoryPtr repo  = DoQuBook::ApplicationContext::instance()->getRepository(repoName);
    \endcode
  * \section usingPlugin Using a plugin
  * \section createPlugin Creating a plugin
  *     see page \ref createAPlugin
  *
  */

namespace DoQuBook
{

    class Book;
    typedef QSharedPointer<Book> BookPtr;

    /*! \class Book
         * \brief A book with a name and a list of pages
         * \author { Maxime Daymard, Vincent Rabeux }
         *
         * The class provides access to the name and the page list of the book
         *
         *
         */
    class DoQuBookSHARED_EXPORT Book : public SimpleMarshallable
    {
            Q_OBJECT

            Q_PROPERTY (int id READ getId WRITE setId NOTIFY idChanged)
            Q_PROPERTY (QString name READ getName WRITE setName NOTIFY nameChanged)
            Q_PROPERTY (QList<PagePtr> pages READ getPages WRITE setPages NOTIFY pagesChanged)
            Q_PROPERTY (int version READ getVersion WRITE setVersion NOTIFY versionChanged)

        public:
            MARSHALLABLE_CREATE_FONCTIONS (Book)


            explicit Book (QString name = "");

            QString getName() const {
                return _name;
            }
            void setName (QString name) {
                _name = name;
            }
            QString getFilePath() const {
                return _filePath;
            }
            void setFilePath (QString filePath) {
                _filePath = filePath;
            }

            PagePtr getPage (int pageNum) const;

            PagePtr findPage (const PagePtr page) const;

            PagePtr findPageByName (const PagePtr page) const;

            PagePtr findPageById (int id) const;

            int findPageIndex (const PagePtr page) const;

            QList<PagePtr> getPages() const {
                return _pages;
            }

            void removePage (PagePtr page);
            void removePage (int pageNum);

            bool equals (const Book & autre) {
                bool equal = autre._pages.size() == _pages.size();

                int i = 0;
                while (equal && i < _pages.size() ) {
                    bool found = false;
                    int j = 0;
                    while (!found && j < autre._pages.size() ) {
                        found = _pages.at (i)->equals (* (autre._pages.at (j) ) );
                        j++;
                    }
                    equal = found;
                    i++;
                }
                return equal && _name == autre._name;
            }

            int getVersion() const {
                return m_version;
            }

            int getId() const {
                return m_id;
            }

        public slots :
            void setPages (QList<PagePtr> pages);

            void addPage (PagePtr page);


            void setVersion (int arg) {
                if (m_version != arg) {
                    m_version = arg;
                    emit versionChanged (arg);
                }
            }

            void setId (int arg) {
                if (m_id != arg) {
                    m_id = arg;
                    emit idChanged (arg);
                }
            }

            virtual void merge (BookPtr book);

        signals:

            void nameChanged (QString arg);

            void pagesChanged (QList<PagePtr> arg);

            void versionChanged (int arg);

            void idChanged (int arg);

            void bookMerged();

        private:
            QString _name;
            QString _filePath;

            MARSHALLABLE_MLIST (QList, Page, pages)
            MARSHALLABLE_ACTIV_1LIST (pages, _pages)
            QList<PagePtr> _pages;

            QString m_name;
            int m_version;
            int m_id;
    };

} // namespace

#endif
