#ifndef FUTURE_HPP
#define FUTURE_HPP

#include <QList>
#include <QObject>

#include <DoQuBook/Page.hpp>
#include <DoQuBook/MarshallableFormater.hpp>
#include <DoQuBook/WebClient.hpp>


template<class T>
class Functor
{

public :
    virtual ~Functor() {}
    virtual void process (QSharedPointer<T>  object) = 0;
};

template<class T>
class Future
{

public :
    virtual ~Future() {}

    virtual QSharedPointer<T> get() = 0;

    virtual void addObserver (Functor<T> *f) = 0;

protected :
    virtual void notifyObservers (QSharedPointer<T>  t) = 0;

};



/**
  http://blog.qt.digia.com/blog/2010/06/17/youre-doing-it-wrong/
  http://qt-project.org/wiki/Threads_Events_QObjects
  */
class TThread : public QThread
{
public :
    TThread() {}
    virtual ~TThread() {}
    void run() {
        connect (this, SIGNAL (finished() ), this, SLOT (deleteLater() ) );
        http->run();
        exec();
    }
    void setWorker (DoQuBook::HttpCallManager * c) {
        http = c;
    }

private:
    DoQuBook::HttpCallManager * http;
};


template<class T>
class FunctorsThread : public QThread
{
public :
    FunctorsThread() {}
    virtual ~FunctorsThread() {}

    void run() {
        connect (this, SIGNAL (finished() ), this, SLOT (deleteLater() ) );
        foreach (Functor<T> *f, observers) {
            f->process (t);
        }
    }
    void addFunctor (Functor<T> *f) {
        observers.append (f);

    }

    void setObject (QSharedPointer<T> o) {
        t = o;
    }

protected :
    QList<Functor<T> *> observers;
    QSharedPointer<T> t;
};



class HttpResponseHandler : public QObject
{
    Q_OBJECT
public :
    HttpResponseHandler() {}
    virtual ~HttpResponseHandler () {}

    virtual void processResult (QByteArray content) = 0;

public slots:

    void setResult (QByteArray content) {
        processResult (content);
    }
};


template<class T>
class EventHandler
{

public :
    virtual ~EventHandler() {}

    virtual void addObserver (Functor<T> *f) = 0;

protected :
    virtual void notifyObservers (QSharedPointer<T>  t) = 0;

};


template <class T>
class HttpEventHandler :  public HttpResponseHandler, public EventHandler <T>
{

public :
    HttpEventHandler (const QString & url)
        :  m_isReady (false),
          manager (NULL),
          http (NULL) {
        m_formater = new MarshallableFormaterJSon<T>;
        manager = new TThread;

        http = new DoQuBook::HttpCallManager (url);
        connect (http, SIGNAL (contentReady (QByteArray) ), this, SLOT (setResult (QByteArray) ) );
    }

    virtual  ~HttpEventHandler () {
        if (manager)  {
            manager->wait();
            delete manager;
        }
        if (http) {
            disconnect (http, SIGNAL (contentReady (QByteArray) ), this, SLOT (setResult (QByteArray) ) );
            delete http;
            http = NULL;
        }
    }


    void initAll() {
        m_isReady = false;

        this->moveToThread (manager);
        http->moveToThread (manager);
        manager->setWorker (http);
    }

    void listen() {
        initAll();
        startHttpManager();
    }

    void startHttpManager() {
        if (!manager->isFinished() ) {
            manager->terminate();
        }
        manager->start();
    }


    virtual void processResult (QByteArray content) {

        try {
            QSharedPointer<T> object;
            if (!content.isEmpty() ) {
                object = m_formater->unMarshall (content);
            }

            m_isReady = true;
            notifyObservers (object);
        }
        catch (DoQuBook::CouldNotParseJSONException e) {
            qCritical() << e.getFailedJSONString();
        }
    }

    void addObserver (Functor<T> *f) {
        functors.append (f);
    }

protected :
    void notifyObservers (QSharedPointer<T>  t) {
        FunctorsThread<T> *functorsThreads = new FunctorsThread<T>;
        foreach (Functor<T> *f, functors) {
            functorsThreads->addFunctor (f);
        }
        functorsThreads->setObject (t);
        functorsThreads->start();
    }


protected :
    MarshallableFormater<T> *m_formater;

    bool m_isReady;
    TThread * manager;
    DoQuBook::HttpCallManager * http;

    QList< Functor<T>* > functors;
};



template <class T>
class HttpFuture :  public HttpResponseHandler, public Future <T>
{

public :
    HttpFuture()
        :  m_isReady (false) {
        m_formater = new MarshallableFormaterJSon<T>;


    }

    virtual  ~HttpFuture () {}


    void initAll() {
        m_isReady = false;
        manager = new TThread;

        http = new DoQuBook::HttpCallManager ("http://uneurl");
        connect (http, SIGNAL (contentReady (QByteArray) ), this, SLOT (setResult (QByteArray) ) );
        this->moveToThread (manager);
        http->moveToThread (manager);
        manager->setWorker (http);
    }

    void run() {
        initAll();
        startHttpManager();
    }

    void startHttpManager() {

        if (!manager->isFinished() ) {
            qWarning() << "Terminate ThreadManager";
            manager->terminate();
        }

        manager->start();

        qDebug() << "Manager started" << QThread::currentThreadId();
        // Will block until a unlock is done in setResult.
        m_mut.lock();
    }

    QSharedPointer<T> get() {
        // Will block until a unlock is done in setResult.
        m_mut.lock();

        m_mut.unlock();
        return object;
    }

    virtual void processResult (QByteArray content) {
        m_content = content;

        if (!m_content.isEmpty() ) {
            object = m_formater->unMarshall (m_content);
        }
        m_isReady = true;

        qDebug() << "Future result ready in thread " << QThread::currentThreadId();
        notifyObservers (object);
        m_mut.unlock();
    }

    void addObserver (Functor<T> *f) {
        observers.append (f);
    }

protected :
    void notifyObservers (QSharedPointer<T>  t) {
        foreach (Functor<T> *f, observers) {
            f->process (t);
        }
    }

    QList<Functor<T> *> observers;

protected :
    MarshallableFormater<T> *m_formater;
    QSharedPointer<T> object;
    bool m_isReady;
    TThread * manager;
    QMutex m_mut;
    QByteArray m_content;
    DoQuBook::HttpCallManager * http;

};


#endif // FUTURE_HPP
