/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#ifndef MARSHALLABLEFORMATER_HPP
#define MARSHALLABLEFORMATER_HPP

#include <QByteArray>
#include <DoQuBook/Marshallable.hpp>
#include <qjson/serializer.h>
#include <qjson/parser.h>
#include <assert.h>
#include <DoQuBook/Exceptions.hpp>

template<class T>
class MarshallableFormater
{
    public:
        MarshallableFormater () {}

        virtual QSharedPointer<T> unMarshall (const QByteArray & file) = 0;

        virtual void marshall (QSharedPointer<T> m, QString * dest) = 0;

    protected:
};


template<class T>
class MarshallableFormaterJSon : public MarshallableFormater<T>
{
    public :

        void addIgnoredProp (const QString & prop) {
            ignoredProps.append (prop);
        }


        virtual QSharedPointer<T> unMarshall (const QByteArray & file) {
            QJson::Parser parser;
            QSharedPointer<T> superMarshall = T::create();

            bool ok;
            QVariantMap map = parser.parse (file, &ok).toMap();

            if (!ok || map.isEmpty() ) {
                throw DoQuBook::CouldNotParseJSONException ("Failed JSON Parsing", file);
            }

            QList<MarshallablePtr> seeAlso = superMarshall->seeAlso();

            if (!seeAlso.empty() ) {
                foreach (MarshallablePtr marshal, seeAlso) {

                    QString s  = QString (marshal->metaObject()->className() ).toLower();
                    QStringList names = s.split ("::");

                    if (map.contains (names.at (names.length() - 1) ) ) {
                        superMarshall = marshal.dynamicCast<T>();
                    }
                }
            }

            QString entityName = superMarshall->entityName();
            QVariantMap bookMap = map[entityName].toMap();

            superMarshall->from (bookMap);
            return superMarshall;
        }


        void marshall (QSharedPointer<T> m, QString * dest) {
            assert (dest);
            QJson::Serializer serializer;
            //serializer.setIndentMode(QJson::IndentFull);

            QVariantMap map;
            QString className  = QString (m->metaObject()->className() ).toLower();

            QStringList namespaces = className.split ("::");
            m->addIgnoredProperty ("@property.type");
            m->addIgnoredProperty ("objectName");
            foreach (QString s, ignoredProps) {
                m->addIgnoredProperty (s);
            }
            map[namespaces.at (namespaces.length() - 1)] =  m->serialize();

            QByteArray array = serializer.serialize (map);
            dest->append (array);
        }

        QStringList ignoredProps;
};

#endif // MARSHALLABLEFORMATER_HPP
