/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#ifndef PAGE_HPP
#define PAGE_HPP

#include <DoQuBook/DoQuBook_global.hpp>

#include <string>

#include <QObject>
#include <QString>
#include <QList>
#include <QSharedPointer>
#include <QImage>
#include <QVariant>
#include <QApplication>

#include <DoQuBook/Annotation.hpp>
#include <DoQuBook/AnnotationLocal.hpp>
#include <DoQuBook/AnnotationGlobal.hpp>
#include <DoQuBook/Marshallable.hpp>
#include <DoQuBook/SimpleMarshallable.hpp>
#include <DoQuBook/Exceptions.hpp>


#include <DoQuBook/Config.hpp>

#include <iostream>
#include <QUrl>


namespace DoQuBook
{

    class ImageService;

    class Page;
    typedef QSharedPointer<Page> PagePtr;

    /*! \class Page
         * \brief A page with a name and a list of annotations
         *
         * The class provides access to the name and the annotation list of the page
         */
    class DoQuBookSHARED_EXPORT Page : public SimpleMarshallable
    {
            Q_OBJECT

        public:
            friend class Book;
            MARSHALLABLE_CREATE_FONCTIONS (Page)

            explicit Page (QObject * parent = NULL);


            virtual ~Page();

            QList<AnnotationPtr> getAnnotations();

            void setAnnotations (const QList<AnnotationPtr> & annotations);

            /// Image (jpeg, ...)
            QString getImageFilePath() const;

            /// \todo quel diff entre getImageFileName  et getImageFilePath
            QString getImageFileName() const;

            /// JSon, XML..
            QString getFilePath() const;

            /// \throws ImageNotFoundException
            Q_INVOKABLE QImage getImage (int w = -1) const;


            void setImage (QString fileName);

            bool equals (const Page & autre);

            /// Page number in the book
            int getNumber() const;

            /// Page number in the book
            void setNumber (int number);

            /*!
              * Page name in the book
              *  \attention  On DoQuBookWeb Page::name must be unique, so please provide a name like bookName-pageName;
              */
            QString getName() const;

            /// Used by an image provider to find the old image.
            QString getOldName() const;


            /// Page name in the book
            void setName (const QString & name);

            virtual void merge (MarshallablePtr page);

            AnnotationPtr findAnnotation (AnnotationPtr ann);

            int getId() const {
                return m_id;
            }

            int getVersion() {
                return m_version;
            }


            /*!
             *
             * \throws AnnotationCastException
             * \throws AnnotationNotFoundException
             */
            template<class T, class U>
            T getAnnotation (const QString & name) {
                AnnotationPtr ann = getAnnotation (name);
                if (ann.isNull() ) {
                    throw AnnotationNotFoundException (getId(), name);
                }
                T annT  = ann.dynamicCast<U>();
                if (annT.isNull() ) {
                    throw AnnotationCastException (ann->getName() );
                }

                return annT;
            }

        public slots:
            void setId (int arg) {
                if (m_id != arg) {
                    m_id = arg;
                    emit idChanged (arg);
                }
            }

            void setVersion (int version) {
                if (m_version != version) {
                    m_version = version;
                    emit versionChanged();
                }
            }


            AnnotationPtr getAnnotation (const QString & name);

            /*!
             *
             * \throws AnnotationCastException
             * \throws AnnotationNotFoundException
             */
            AnnotationLocalPtr getAnnotationLocal (const QString & name);

            /*!
             *
             * \throws AnnotationCastException
             * \throws AnnotationNotFoundException
             */
            AnnotationGlobalPtr getAnnotationGlobal (const QString & name);


            void addGlobalAnnotation (const QString & name, const QVariant & value, bool isGroundTruth = false);
            void addAnnotatedPolygon (const QString & name, DoQuBook::AnnotatedPolygonPtr poly);
            /// return false if not added
            bool addLocalAnnotation (AnnotationLocalPtr & annLocal);

            void deleteGlobalAnnotation (const QString & name);
            void deleteLocalAnnotation (Defect d, DoQuBook::AnnotatedPolygonPtr poly);

            void removeAnnotation (AnnotationPtr annotation);
            void removeAnnotation (int annotationNum);
            void removeAnnotations();

            bool contains (const QString & name);
            /// Checks if d is local and contains poly
            bool contains (Defect d, DoQuBook::AnnotatedPolygonPtr poly);

            void installImageService (ImageService * srv);


            // To be refactored (Not saved by services)
            void addFeatureSet(const QString &setName, QMap<QString, double> pageFeats);
            QMap<QString, double> getFeatureSet(const QString &setName);
            bool hasFeatureSet(const QString &setName);

        signals:
            void idChanged (int arg);

            void annotationsListChanged();

            void nameChanged();

            void versionChanged();

        private :
            /*! \todo On dois pouvoir récupérer une annotation local et une global !
             * On doit pouvoir la récuperer par son nom et pas par son index !!
            */
            AnnotationPtr getAnnotation (int annotationNum);


        protected :

            AnnotationPtr getAnnotationByName (const QString & name);
            void addAnnotation (AnnotationPtr annotation);

            Q_PROPERTY (int id READ getId WRITE setId NOTIFY idChanged)

            MARSHALLABLE_MLIST (QList, Annotation, annotations)
            Q_PROPERTY (QList<AnnotationPtr> annotations READ getAnnotations WRITE setAnnotations)

            Q_PROPERTY (QString name READ getName WRITE setName NOTIFY nameChanged)
            Q_PROPERTY (int version READ getVersion WRITE setVersion NOTIFY versionChanged)


        protected :
            MARSHALLABLE_ACTIV_1LIST (annotations, _annotations)


        private:
            QString _name;

            int _number;

            QList<AnnotationPtr> _annotations;

            QUrl annotationsLink;

            /// example : image.jpeg
            QString _imageFileName;

            /// example : c:/.../image.jpeg
            QString _imageFilePath;

            /// example : c:/.../page.json
            QString _filePath;


            ImageService * m_imageService;

            /// Used by an image provider to find the old image.
            QString oldName;

            int m_version;

            int m_id;

            QMap< QString, QMap<QString, double> > featureSets;

    };

} // namespace

Q_DECLARE_METATYPE ( DoQuBook::PagePtr )

#endif // PAGE_HPP
