#ifndef PLUGINLOADER_HPP
#define PLUGINLOADER_HPP

#include <DoQuBook/DoQuBook_global.hpp>

#include <DoQuBook/Page.hpp>
#include <typeinfo>
//#include <cxxabi.h>



class QApplication;
#include <DoQuBook/AlgorithmOnTask.hpp>
#include <QPluginLoader>
#include <DoQuBook/DBWebBinarizerRemoteProxy.hpp>
#include <DoQuBook/DBWebMeasurerRemoteProxy.hpp>
#include <DoQuBook/DBWebOCRRemoteProxy.hpp>



namespace DoQuBook
{

    /**
     * @brief The LocalPluginFactory class, loads a plugin with a file path (ie: /usr/.../libPlugin.so)
     * using QPluginLoader. The plugin is unloaded with a shared_ptr deleter PluginDeleter.
     * \code
     *      LocalPluginFactory facto("path/to/libPlugin.so");
     *      std::tr1::shared_ptr<QObject> p (facto.build(), PluginDeleter(facto));
     * \endcode
     */
    class LocalPluginFactory : public PluginFactory
    {
        public :
            LocalPluginFactory (const QString & filePath)
                : m_loader (filePath)
            {
              qCritical() << "Creating QPluginLoader on " << filePath;
            }

            virtual ~LocalPluginFactory () {}
            virtual QObject * build() {
                qCritical() << "Plugin loading";
                QObject * plugin  = m_loader.instance();
                return plugin;
            }

            virtual void unLoad() {
                qCritical() << "Plugin is loaded ?  " << m_loader.isLoaded();
                qCritical() << "Plugin will unload";
                bool unloaded = m_loader.unload();
                qCritical() << "Plugin unloaded ? " << unloaded;
            }

        protected :
            QPluginLoader m_loader;
    };


    class DoQuBookSHARED_EXPORT PluginLoader : public DoQuBook::Singleton<PluginLoader>
    {
            friend class DoQuBook::Singleton<PluginLoader>;
        public:

            void loadLocalPlugins();
            void loadPlugins();
            void loadRemoteProxyPLugins ();



            void unLoadPlugin (const QString & name) {
                _plugins[name]->unLoad();
            }

            template <typename T>
            typename std_shared_ptr<T>::type getPlugin (const QString & name) {
                if (!_pluginsLoaded) {
                    loadPlugins();
                }

                typename std_shared_ptr<T>::type p;
                if (_plugins.contains (name) ) {
                    PluginFactory * facto = _plugins[name];
                    // TODO: le deleter !
                    PluginDeleter deleter (facto);
                    std_shared_ptr<QObject>::type qObjectPlugin (facto->build(),  deleter);
#ifdef HAVE_CPP11
                    p = std::dynamic_pointer_cast<T> (qObjectPlugin);
#else
                    p = std::tr1::dynamic_pointer_cast<T> (qObjectPlugin);
#endif
                }
                else {
                    throw CouldNotFindPluginException(name);
#ifdef DEBUG_LOG
                    qCritical() << "could not find plugin named " << name;
                    qCritical() << "Plugin list with requested type : " << listPlugins<T>();
#endif
                }
                return p;
            }

            template <class T>
            QStringList listPlugins() {
                if (!_pluginsLoaded) {
                    loadPlugins();
                }

                QStringList pluginNames;
                QMapIterator <QString, PluginFactory * > it (_plugins);
                while (it.hasNext() ) {
                    it.next();
                    PluginFactory * facto = _plugins[it.key()];
                    std_shared_ptr<QObject>::type qObjectPlugin (facto->build(), PluginDeleter (facto) );
#ifdef HAVE_CPP11
                    typename std_shared_ptr<T>::type p = std::dynamic_pointer_cast<T> (qObjectPlugin);
#else
                    typename std_shared_ptr<T>::type p = std::tr1::dynamic_pointer_cast<T> (qObjectPlugin);
#endif
                    if (p) {
                        pluginNames.append (it.key() );
                    }

                }

                return pluginNames;
            }

            //QList<BinarizerPtr> binarizers();
            QMap<QString, std_shared_ptr<AlgorithmOnTask>::type > algorihmsOnTaskImageResource (bool loadRemotePlugins = false);

            void setPluginDirPath (const QString & pathToPlugins);
        private :
            PluginLoader();


        private :
            QMap<QString, PluginFactory * > _plugins;

            QString _pathToPlugins;
            bool _pluginsLoaded;

    };
}

#endif // PLUGINLOADER_HPP
