#ifndef REGION_HPP
#define REGION_HPP

#include <DoQuBook/Point.hpp>
#include <QImage>
#include <vector>
#include <DoQuBook/Config.hpp>

namespace DoQuBook
{

    class Region;
    typedef QSharedPointer<Region> RegionPtr;
    typedef QList<RegionPtr> RegionPtrList;

    class Region
    {
        public:
            Region();
            Region (const QList<PointPtr> &domain);
            Region (const QString & label);
            Region (const QList<PointPtr> &domain, const QString & label);
            Region (const std::vector<PointPtr> &domain);

            QString getLabel();
            void setLabel (const QString & label);

            QList<PointPtr> getContour();
            //void setContour(const QList<PointPtr> &contour);
            void addPointToContour (QPointF p);

            void addPointToDomain (const PointPtr & pt);

            QRectF  boundingBox();
            void setBoundingBox (const PointPtr & p1, const PointPtr & p2);

            double area();
            void setArea (double a);

            double perimeter();
            void setPerimeter (double p);

            enum DrawingMode { BBox, Contour, Domain };
            QImage thumbnail (const QImage & image);

            void draw (QImage * image, const QColor & color, DrawingMode drawingMode = Domain);

            // All Points
            QList<PointPtr> domain() const;

            double meanIntensity (const QImage & image) const;

            static double meanIntensity (QList<Region> &regions, const QImage & image);


        private :
            QPolygonF _contour;


            QString _label;

            /// Memoized field;
            QList<DoQuBook::PointPtr> _domain;
            QRectF _bbox;
            double _area;
            double _perimeter;
    };


    class RegionPointProcessor;
    typedef QSharedPointer<RegionPointProcessor> RegionPointProcessorPtr;

    class RegionPointProcessor
    {
        public :
            RegionPointProcessor() {}
            virtual ~RegionPointProcessor() {}

            virtual void processPoint (int x, int y) = 0;
            virtual void newRegion() = 0;
            virtual void endRegion() = 0;
            virtual void boundingBoxOfRegion (int x1, int y1, int x2, int y3) = 0;

            /// can be used as filters
            virtual void area (double a) = 0;
            virtual void perimeter (double p) = 0;
    };

    class BoundingBoxProcessor : public RegionPointProcessor
    {
        public :
            BoundingBoxProcessor() {}
            virtual ~BoundingBoxProcessor() {}

            virtual void processPoint (int /*x*/, int /*y*/)
            {}

            virtual void newRegion()  {}
            virtual void endRegion() {}

            virtual void boundingBoxOfRegion (int x1, int y1, int x2, int y2) {
                QPointF topLeft (x1, y1);
                QPointF bottomRight (x2, y2);
                QRectF bbox (topLeft, bottomRight);
                process (bbox);
            }

            virtual void process (QRectF & bbox) = 0;

            virtual void area (double ) {};
            virtual void perimeter (double ) {};

    };

    class RegionPainter : public RegionPointProcessor
    {
        public :
            RegionPainter (QImage & image) : _image (image), regionAccepted (false) {}
            virtual ~RegionPainter() {}

            virtual void processPoint (int x, int y) {
                if (regionAccepted) {
                    _image.setPixel (x, y, qRgb (255, 0, 0) );
                }
            }

            virtual void newRegion() {}
            virtual void endRegion() {}
            virtual void boundingBoxOfRegion (int /*x1*/, int /*y1*/, int /*x2*/, int /*y3*/) {}

            /// can be used as filters
            virtual void area (double a) {
                regionAccepted = a >= 20;
            }
            virtual void perimeter (double /*p*/) {}

        protected :
            QImage & _image;
            bool regionAccepted;
    };


}
#endif // REGION_HPP
