/*!
 * \brief This file is part of DoQuBook Framework
 *
 * Main author :  Rabeux Vincent <rabeux@labri.fr>
 * from LaBRI, University of Bordeaux 1 - Sud Ouest
 *
 * \details DoQuBook is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * \version RC-0.0.2
 * \date    2011
 *
 * Authors and main contributors :
 * \author Rabeux Vincent <rabeux@labri.fr>
 * \author Maxime Daymard <mr.xyz@hotmail.fr>
 * \author Jean-Philippe Domenger <domenger@labri.fr>
 * \author Nicholas Journet <journet@labri.fr>
 *
 * \copyright GNU LGPL.
 * Copyright (C) 2003-2011 <LaBRI>. All Rights Reserved.
 * Web: <www.labri.fr>
 *
 * DoQuBook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
#ifndef REGISTRY_HPP
#define REGISTRY_HPP

#include <DoQuBook/Singleton.hpp>
#include <DoQuBook/Factory.hpp>
#include <QHash>
#include <QDebug>
#include <typeinfo>
#include <cxxabi.h>
#include <QStringList>
#include <DoQuBook/DoQuBook_global.hpp>

namespace DoQuBook
{

    /**
      * \brief This class defines a templated Registry.
      *  http://matthieu-brucher.developpez.com/tutoriels/conception/registry/
      *
      * This Registry will in fact register factories. A factory has to implement 2 methods and can extend Factory  :
      * \code
          virtual SharedObject create() = 0;
          virtual QString name() = 0;
        \endcode
        Where SharedObject is the Object that the registry will retreive by its name or class name.
      *
      * Usage :
      * \code
            class MyRegistry : public DoQuMeasures::Registry<SharedObjectClassPtr, SharedObjectClassFactory>
            {
            private :
                MyRegistry();

            };
        \endcode
      *
      * How to use MyRegistry ?
      * \code
        ...
            MyRegistry::instance()->registerClass<ImplSharedObjClassFactory>();
            MyRegistry::instance()->get<ImplSharedObjClassPtr>();
        ...
        \endcode

        Please consider RegistrationMethodRegistry that implements Registry and defines MACROS in order to create factories at compilation time.
      */
    template <class Class, class Factory>
    class DoQuBookSHARED_EXPORT Registry
    {
            friend class Singleton<Registry >;

        public :

            Class get (const QString & name) {
                if (!m_factories.contains (name) ) {
                    QString message = name + " has not been registered";
                    qCritical() << message;
                    qCritical() << "Registered objects are : " << registered();
                    throw message;
                }
                return m_factories[name]->create (name);
            }

            template<class T>
            Class get() {
                int     status;
                char  * realname;
                realname = abi::__cxa_demangle (typeid (T).name(), 0, 0, &status);
                return get (realname);
            }

            virtual ~Registry() {}

            void add (Factory * facto) {
                if (m_factories.contains (facto->name() ) ) {
                    delete facto;
                }
                else {
                    m_factories[facto->name()] = facto;
                }
            }

            QStringList registered() {
                QStringList list;
                foreach (QString key, m_factories.keys() ) {
                    list.append (key);
                }
                return list;
            }

            template <class T>
            bool registerClass() {
                add (new T);
                return true;
            }

        protected :
            Registry() {}
            typedef QHash<QString, Factory *> MapFactories;
            MapFactories m_factories;

    };
}


#define DECLARE_FACTORY(T) const static bool REGISTERED_WITH_FACTORY
#define INIT_FACTORY_ON_REGISTRY(T, R) const bool T::REGISTERED_WITH_FACTORY = R::instance()->registerClass<T>();


#endif // REGISTRY_HPP
