#ifndef REMOTEPLUGINFACTORY_H
#define REMOTEPLUGINFACTORY_H

#include <QObject>
#include <DoQuBook/Repository.hpp>
#include <DoQuBook/Executor.hpp>

namespace DoQuBook
{

    /**
     * @brief The PluginFactory class build or unLoad a plugin \see LocalPluginFactory and \see RemotePluginFactory
     */
    class PluginFactory
    {
        public :
            virtual ~PluginFactory () {}
            virtual QObject * build() = 0;
            virtual void unLoad() = 0;
    };

    /**
     * @brief The PluginDeleter class, is a functor given to a shared_ptr and calls the given PluginFactory to unLoad it.
     * The PluginFactory is indeed the only one to how the plugin was loaded.
     *
     * \code
     *      LocalPluginFactory facto("path/to/libPlugin.so");
     *      std::tr1::shared_ptr<QObject> p (facto.build(), PluginDeleter(facto));
     * \endcode
     */
    class PluginDeleter
    {
        public:
            PluginDeleter (PluginFactory * facto)
                : m_facto (facto) {}
            virtual void operator () (QObject * /*d*/) const {
                qCritical() << "PluginDeleter::operator()";
                //m_facto->unLoad();
            }
        private :
            PluginFactory * m_facto;
    };

    /**
     * @brief The RemotePluginFactory class builds a plugin proxy (ie: distant plugins).
     * It needs to be implemented to specify whitch plugin to build \see DBWebBinarizerRemoteProxy::BinarizerRemotePluginFactory
     */
    class RemotePluginFactory : public PluginFactory
    {
        public :
            RemotePluginFactory() {}

            virtual ~RemotePluginFactory() {}

            virtual bool accepts (RepositoryPtr repo, ExecutorPtr exec) = 0;

            void setup (RepositoryPtr repo, ExecutorPtr exec) {
                m_repo = repo;
                m_exec = exec;
            }

            virtual QObject * build() = 0;
            virtual void unLoad()  = 0;
            virtual RemotePluginFactory * clone() = 0;

        protected :
            RepositoryPtr m_repo;
            ExecutorPtr m_exec;
    };

}


#endif // REMOTEPLUGINFACTORY_H
