#ifndef QVARIANTFROMTASKPARAMETER_HPP
#define QVARIANTFROMTASKPARAMETER_HPP

#include <QSharedPointer>
#include <DoQuBook/Task.hpp>
#include <DoQuBook/RESTImageManager.hpp>
#include <DoQuBook/Exceptions.hpp>
#include <DoQuBook/Repository.hpp>
#include <QUrl>
#include <QDebug>

#include <DoQuBook/Histogram.hpp>
#include <DoQuBook/RESTObjectManager.hpp>


namespace DoQuBook
{

    class RemoteQVariantConverter
    {
        public:
            RemoteQVariantConverter();
            virtual  ~RemoteQVariantConverter() {}

            virtual bool accept (const QVariant & variant) = 0;

            virtual bool accept (const QString & type) = 0;
            /**
             * Converts a QString such as url or double or int or date or any QVariant "simple" types, to a real QVariant.
             * @brief toVariant
             * @param urlOrValue
             * @return
             */
            virtual QVariant toVariant (const QString & urlOrValue, TaskPtr task) = 0;
            /**
             * Converts any complex object that can be stored in a QVariant to a String.
             * @brief toString
             * @param variant
             * @return
             */
            virtual QString toString (const QVariant & variant, TaskPtr task) = 0;

            /*
            if(variant.canConvert<AnnotationGlobalPtr>()) {
                setResourceType("DoQuBook::AnnotationGlobal");
            }

            if(variant.canConvert<PagePtr>()) {
                setResourceType("Page");
            }
            qDebug() << "Sored result type = " << getResourceType();
            qDebug() << "Variant type name  = " << variant.typeName();
               */
            virtual QString typeName () = 0;
    };

    typedef QSharedPointer<RemoteQVariantConverter> RemoteQVariantConverterPtr;

    class RemoteQVariantImageConverter : public RemoteQVariantConverter
    {
        public:
            RemoteQVariantImageConverter (TaskImageService & tempImageService);
            virtual  ~RemoteQVariantImageConverter() {}

            virtual bool accept (const QString & type) {
                return type == "Image";
            }

            virtual bool accept (const QVariant & variant) {
                return variant.type() == QVariant::Image;
            }

            virtual QVariant toVariant (const QString & urlOrValue, TaskPtr /*task*/) {
                QString url = urlOrValue;
                qDebug() << "RemoteQVariantImageConverter :: Retreive Image at ; " << url;

                DoQuBook::RESTImageManager manager (DoQuBook::RESTImageManager::JPEG);
                manager.addRessourcePath (DoQuBook::WebClient::GET, url);

                QMap<QString, QString> params;
                QImage param = manager.get (params);


                return param;
            }
            virtual QString toString (const QVariant & variant, TaskPtr task) {
                QUrl urlP = m_taskImageService.add (variant.value<QImage>(), task);
                return urlP.toString();
            }

            virtual QString typeName () {
                return "Image";
            }

        protected :
            TaskImageService & m_taskImageService;
    };

    typedef QSharedPointer<RemoteQVariantImageConverter> RemoteQVariantImageConverterPtr;

    class RemoteQVariantStringConverter : public RemoteQVariantConverter
    {
        public:
            RemoteQVariantStringConverter () {}
            virtual  ~RemoteQVariantStringConverter () {}

            virtual bool accept (const QString & type) {
                qDebug() << "RemoteQVariantStringConverter =: type =:" << type ;
                return type == "String";
            }

            virtual bool accept (const QVariant & variant) {
                return variant.type() == QVariant::String;
            }

            virtual QVariant toVariant (const QString & urlOrValue, TaskPtr /*task*/) {
                return urlOrValue;
            }

            virtual QString toString (const QVariant & variant, TaskPtr /*task*/) {
                return variant.toString();
            }

            virtual QString typeName () {
                return "String";
            }
    };

    class RemoteQVariantNumberConverter : public RemoteQVariantConverter
    {
        public:
            RemoteQVariantNumberConverter () {}
            virtual  ~RemoteQVariantNumberConverter () {}

            virtual bool accept (const QString & type) {
                return type == "Number";
            }

            virtual bool accept (const QVariant & variant) {
                return variant.type() == QVariant::Double
                       || variant.type() == QVariant::Int
                       || variant.type() == QVariant::UInt
                       || variant.type() == QVariant::ULongLong ;
            }

            virtual QVariant toVariant (const QString & urlOrValue, TaskPtr /*task*/) {
                return urlOrValue;
            }

            virtual QString toString (const QVariant & variant, TaskPtr /*task*/) {
                return variant.toString();
            }

            virtual QString typeName () {
                return "Number";
            }
    };


    class RemoteQVariantBooleanConverter : public RemoteQVariantConverter
    {
        public:
            RemoteQVariantBooleanConverter () {}
            virtual  ~RemoteQVariantBooleanConverter () {}

            virtual bool accept (const QString & type) {
                return type == "Boolean";
            }

            virtual bool accept (const QVariant & variant) {
                return variant.type() == QVariant::Bool;
            }

            virtual QVariant toVariant (const QString & urlOrValue, TaskPtr /*task*/) {
                return urlOrValue;
            }

            virtual QString toString (const QVariant & variant, TaskPtr /*task*/) {
                return variant.toString();
            }

            virtual QString typeName () {
                return "Boolean";
            }
    };

    //sumOfHeight;368|sumOfWidth;1381
    class RemoteQVariantNumberMapConverter : public RemoteQVariantConverter
    {
        public:
            RemoteQVariantNumberMapConverter () {}
            virtual  ~RemoteQVariantNumberMapConverter () {}

            virtual bool accept (const QString & type) {
                return type == "NumberMap";
            }

            virtual bool accept (const QVariant & variant) {
                bool isMap = variant.type() == QVariant::Map
                             && variant.toMap().size() > 0 ;
                if (isMap) {
                    QVariant element = variant.toMap().values().at (0);
                    bool isNumber = element.type() == QVariant::Double
                                    || element.type() == QVariant::Int
                                    || element.type() == QVariant::UInt
                                    || element.type() == QVariant::ULongLong ;
                    return isNumber;
                }
                else {
                    return false;
                }


            }

            virtual QVariant toVariant (const QString & urlOrValue, TaskPtr /*task*/) {
                QMap<QString, QVariant> measures;
                QStringList sList = urlOrValue.split ("|");
                foreach (QString s, sList) {
                    QStringList listValues = s.split (";");
                    measures[listValues.at (0)] = listValues.at (1);
                }
                return measures;
            }

            virtual QString toString (const QVariant & variant, TaskPtr /*task*/) {
                QString s;

                QVariantMap map = variant.toMap();

                QMapIterator<QString, QVariant> it (map);
                int i = 0;
                while (it.hasNext() && i < map.size() - 1) {
                    it.next();
                    QVariant value = it.value();
                    QString k = it.key();

                    s.append (k);
                    QString val = value.toString();
                    s.append (";");
                    s.append (val);
                    s.append ("|");
                    i++;
                }
                it.next();
                QVariant value = it.value();
                QString k = it.key();
                s.append (k);
                QString val = value.toString();
                s.append (";");
                s.append (val);

                return s;
            }

            virtual QString typeName () {
                return "NumberMap";
            }
    };

    // url1 | url2 | url3 ...
    class RemoteQVariantImageListConverter : public RemoteQVariantConverter
    {
        public:

            RemoteQVariantImageListConverter (TempImageService & tempImageService)
                : m_tempImageService (tempImageService) {}

            virtual  ~RemoteQVariantImageListConverter () {}

            virtual bool accept (const QString & type) {
                return type == "ImageList";
            }

            virtual bool accept (const QVariant & variant) {
                return variant.type() == QVariant::List
                       && variant.toList().size() > 0
                       && variant.toList().at (0).type() == QVariant::Image;
            }

            virtual QVariant toVariant (const QString & urlOrValue, TaskPtr /*task*/) {
                QList<QVariant> list;
                QStringList sList = urlOrValue.split ("|");
                foreach (QString s, sList) {
                    QVariant v = getImage (s.trimmed() );
                    list << v;
                }
                return list;
            }

            /**
             * wil form a string such as :
             * http://monserveur.com/32 | http://monserveur.com/33
             * @brief toString
             * @param variant
             * @return
             */
            virtual QString toString (const QVariant & variant, TaskPtr /*task*/) {
                QString result;
                QList<QVariant> list = variant.toList();
                QListIterator<QVariant> it (list);
                int i = 0;
                while (it.hasNext() && i < list.size() - 1) {
                    QString url = pushImage (it.next() );
                    result.append (url);
                    result.append (" | ");
                    i++;
                }
                // the last one
                QString url = pushImage (it.next() );
                result.append (url);

                return result;
            }

            virtual QString typeName () {
                return "ImageList";
            }

        private :
            QString pushImage (const QVariant & v) {
                QUrl urlP = m_tempImageService.add (v.value<QImage>() );
                return urlP.toString();
            }

            QVariant getImage (const QString & url) {

                DoQuBook::RESTImageManager manager (DoQuBook::RESTImageManager::JPEG);
                manager.addRessourcePath (DoQuBook::WebClient::GET, url);

                QMap<QString, QString> params;
                QImage param = manager.get (params);

                return param;
            }

        protected :
            TempImageService & m_tempImageService;
    };


    class RemoteQVariantHistoGramConverter : public RemoteQVariantConverter
    {

        public :
            RemoteQVariantHistoGramConverter (QString serverAdress)
                : m_histogramManager (WebClient::JSON) {

                QString postUrl (serverAdress + "/DoQuBookWeb/rest/task/add/histogram/(taskId)");
                m_histogramManager.addRessourcePath (WebClient::POST, postUrl);
            }

            virtual  ~RemoteQVariantHistoGramConverter() {}

            virtual bool accept (const QString & type) {
                return type == "Histogram";
            }

            virtual bool accept (const QVariant & variant) {
                return variant.canConvert<DoQuBook::HistogramPtr>();
            }

            virtual QVariant toVariant (const QString & urlOrValue, TaskPtr /*task*/) {
                // doxnload
                m_histogramManager.addRessourcePath (WebClient::GET, urlOrValue);
                QMap<QString, QString> params;
                HistogramPtr histo = m_histogramManager.get (params);
                return QVariant::fromValue<DoQuBook::HistogramPtr> (histo);
            }

            virtual QString toString (const QVariant & variant, TaskPtr task) {
                // Upload
                QMap<QString, QString> params;
                params["taskId"] = QString::number (task->getId() );
                HistogramPtr histo =  variant.value<HistogramPtr>();
                QByteArray array = m_histogramManager.postAndReturnByteArray (histo, params);
                QUrl url (array);
                return url.toString();
            }

            virtual QString typeName () {
                return "Histogram";
            }
        private :
            RESTObjectManager<Histogram> m_histogramManager;
    };

}

#endif // QVARIANTFROMTASKPARAMETER_HPP
