#ifndef TASK_HPP
#define TASK_HPP

#include <QString>
#include <DoQuBook/DoQuBook_global.hpp>
#include <QSharedPointer>
#include <QVariant>
#include <DoQuBook/SimpleMarshallable.hpp>
#include <DoQuBook/TempImageService.hpp>
#include <DoQuBook/AnnotationGlobal.hpp>


namespace DoQuBook
{

    class Task;
    typedef QSharedPointer<Task> TaskPtr;

    class TaskParameter;
    typedef QSharedPointer<TaskParameter> TaskParameterPtr;

    class DoQuBookSHARED_EXPORT TaskParameter : public SimpleMarshallable
    {
            Q_OBJECT

            Q_PROPERTY (int id READ getId WRITE setId)
            Q_PROPERTY (QString name READ getName WRITE setName)
            Q_PROPERTY (QVariant value READ getValue WRITE setValue)
            Q_PROPERTY (QString parameterType READ getParameterType WRITE setParameterType)

        public :
            TaskParameter();
            virtual ~TaskParameter();

            MARSHALLABLE_CREATE_FONCTIONS (TaskParameter)
            static TaskParameterPtr create (const QString & name,
                                            QUrl value,
                                            const QString & type);
            static TaskParameterPtr create (const QString & name,
                                            QImage value
                                           );
            static TaskParameterPtr create (const QString & name,
                                            QVariant value
                                           );
            static TaskParameterPtr create (const QString & name,
                                            double value);
            static TaskParameterPtr create (const QString & name,
                                            int value);
            static TaskParameterPtr create (const QString & name,
                                            QString value);



            QString getName() const {
                return m_name;
            }

            QVariant getValue() const {
                return m_value;
            }

            QString getParameterType() const {
                return m_parameterType;
            }

            int getId() const {
                return m_id;
            }

        public slots:
            void setName (QString arg) {
                m_name = arg;
            }

            void setValue (QVariant arg) {
                m_value = arg;
            }

            void setParameterType (QString arg) {
                m_parameterType = arg;
            }
            void setId (int arg) {
                m_id = arg;
            }

        private :

            TaskParameter (const QString & name,
                           QUrl value,
                           const QString & type
                          );
            TaskParameter (const QString & name,
                           QImage value);
            TaskParameter (const QString & name,
                           QVariant value);
            TaskParameter (const QString & name,
                           double value);
            TaskParameter (const QString & name,
                           int value);
            TaskParameter (const QString & name,
                           QString value);



        private :
            int m_id;
            QString m_name;
            QVariant m_value;
            QString m_parameterType;

    };


    class DoQuBookSHARED_EXPORT Task :  public SimpleMarshallable
    {

            Q_OBJECT

            Q_PROPERTY (int id READ getId WRITE setId)
            Q_PROPERTY (QString userMail READ getUserMail WRITE setUserMail)

            Q_PROPERTY (int date READ getDate WRITE setDate)
            Q_PROPERTY (int span READ getSpan WRITE setSpan)
            Q_PROPERTY (QString status READ getStatusAsString WRITE setStatusAsString)

            Q_PROPERTY (QString errorMessage READ getErrorMessage WRITE setErrorMessage)

            Q_PROPERTY (QString executorId READ getExecutorID WRITE setExecutorID)
            Q_PROPERTY (QString resourceType READ getResourceType WRITE setResourceType)
            Q_PROPERTY (QVariant result READ getResult WRITE setPureResult)
            Q_PROPERTY (QList<TaskParameterPtr> params READ getParams WRITE setParams)

        public:

            enum TaskStatus {
                Building,
                New,
                Done,
                Fail,
                Accepted,
                RefreshForm
            };

            static QMap<TaskStatus, QString> taskStatusString;

            struct DoQuBookSHARED_EXPORT taskStatusStringInit {
                taskStatusStringInit();
            };
            static taskStatusStringInit initer;

            MARSHALLABLE_CREATE_FONCTIONS (Task)

            static TaskPtr create (const QString & executorID, const QVariantMap & paramsMap = QVariantMap() );

            MARSHALLABLE_MLIST (QList, TaskParameter, params)
            MARSHALLABLE_ACTIV_1LIST (params, m_params)

            Task();
            virtual ~Task() {}

            int getId() const {
                return m_id;
            }

            QString getUserMail() const {
                return m_userMail;
            }

            int getDate() const {
                return m_date;
            }

            int getSpan() const {
                return m_span;
            }

            QString getStatusAsString() const {
                return m_status;
            }

            TaskStatus getStatus() const {
                bool found = false;
                QMapIterator<TaskStatus, QString> it (taskStatusString);
                while (it.hasNext() && !found) {
                    it.next();
                    found = it.value() == m_status;
                }
                if (found) {
                    return it.key();
                }
                else {
                    return New;
                }
            }

            QVariant getResult() const {
                return m_result;
            }

            QList<TaskParameterPtr> getParams() const {
                return m_params;
            }

            TaskParameterPtr getParam (const QString & name);

            bool hasParam (const QString & name);

            QString getExecutorID() const {
                return m_executorId;
            }

            QString getResourceType() const {
                return m_resourceType;
            }

            QString getErrorMessage() const {
                return m_errorMessage;
            }

        public slots:
            void setId (int arg) {
                m_id = arg;
            }

            void setUserMail (QString arg) {
                m_userMail = arg;
            }
            void setDate (int arg) {
                m_date = arg;
            }
            void setSpan (int arg) {
                m_span = arg;
            }
            void setStatus (TaskStatus arg) {
                m_status = taskStatusString[arg];
            }
            void setStatusAsString (const QString & arg) {
                if (taskStatusString.values().contains (arg) ) {
                    m_status = arg;
                }
            }

            void setPureResult (const QVariant & variant);
            QVariant getPureResult();

            void setResult (const QVariant & variant);

            void setParams (QList<TaskParameterPtr> arg) {
                m_params = arg;
            }

            void addParam (TaskParameterPtr arg) {
                m_params.append (arg);
            }

            void addParam (const QString & name, const QVariant & v);
            void setParam (const QString & name, const QVariant & v);
            void setParam (TaskParameterPtr arg);

            void setExecutorID (QString arg) {
                m_executorId = arg;
            }

            void setResourceType (QString arg) {
                m_resourceType = arg;
            }

            void setErrorMessage (QString arg) {
                m_errorMessage = arg;
            }

        private :
            int m_id;
            QString m_userMail;

            int m_date;

            int m_span;

            QString m_status;

            QVariant m_result;

            QList<TaskParameterPtr> m_params;
            QString m_executorId;
            QString m_resourceType;
            QString m_errorMessage;
    };


}

#endif // TASK_HPP
